@@ -53,6 +53,28 @@ static char *find_modeline(char *modeString, char *pModeLines,
 
 
 
+static void print_display_name(Display *dpy, int target_id, int attr,
+                               char *name)
+{
+    Bool ret;
+    char *str;
+
+    ret = XNVCTRLQueryTargetStringAttribute(dpy,
+                                            NV_CTRL_TARGET_TYPE_DISPLAY,
+                                            target_id, 0,
+                                            attr,
+                                            &str);
+    if (!ret) {
+        printf("    %18s : N/A\n", name);
+        return;
+    }
+
+    printf("    %18s : %s\n", name, str);
+    XFree(str);
+}
+
+
+
 
 int main(int argc, char *argv[])
 {
@@ -363,8 +385,48 @@ int main(int argc, char *argv[])
         
         XFree(str);
     }
-    
-    
+
+
+    /*
+     * query the MetaModes for the X screen, using
+     * NV_CTRL_BINARY_DATA_METAMODES_VERSION_2.
+     */
+
+    else if (strcmp(argv[1], "--print-metamodes-version2") == 0) {
+
+        /* get list of metamodes */
+
+        ret = XNVCTRLQueryBinaryData(dpy, screen, 0, // n/a
+                                     NV_CTRL_BINARY_DATA_METAMODES_VERSION_2,
+                                     (void *) &str, &len);
+
+        if (!ret) {
+            fprintf(stderr, "Failed to query MetaModes.\n\n");
+            return 1;
+        }
+
+        /*
+         * the returned data is in the form:
+         *
+         *   "MetaMode 1\0MetaMode 2\0MetaMode 3\0Last MetaMode\0\0"
+         *
+         * so walk from one "\0" to the next to print each MetaMode.
+         */
+
+        printf("MetaModes:\n");
+
+        start = str;
+        for (j = 0; j < len; j++) {
+            if (str[j] == '\0') {
+                printf("  %s\n", start);
+                start = &str[j+1];
+            }
+        }
+
+        XFree(str);
+    }
+
+
     /*
      * query the currently in use MetaMode.  Note that an alternative
      * way to accomplish this is to use XRandR to query the current
@@ -387,8 +449,32 @@ int main(int argc, char *argv[])
 
         XFree(str);
     }
-    
-    
+
+
+    /*
+     * query the currently in use MetaMode.  Note that an alternative
+     * way to accomplish this is to use XRandR to query the current
+     * mode's refresh rate, and then match the refresh rate to the id
+     * reported in the returned NV_CTRL_BINARY_DATA_METAMODES_VERSION_2 data.
+     */
+
+    else if (strcmp(argv[1], "--print-current-metamode-version2") == 0) {
+
+        ret = XNVCTRLQueryStringAttribute(dpy, screen, mask,
+                                          NV_CTRL_STRING_CURRENT_METAMODE_VERSION_2,
+                                          &str);
+
+        if (!ret) {
+            fprintf(stderr, "Failed to query the current MetaMode.\n\n");
+            return 1;
+        }
+
+        printf("current metamode: \"%s\"\n\n", str);
+
+        XFree(str);
+    }
+
+
     /*
      * add the given MetaMode to X screen's list of MetaModes, using
      * NV_CTRL_STRING_OPERATION_ADD_METAMODE; example MetaMode string:
@@ -791,46 +877,45 @@ int main(int argc, char *argv[])
     
     
     /*
-     * query the TwinViewXineramaInfoOrder
+     * query the nvidiaXineramaInfoOrder
      */
     
-    else if (strcmp(argv[1], "--query-twinview-xinerama-info-order") == 0) {
+    else if (strcmp(argv[1], "--query-nvidia-xinerama-info-order") == 0) {
         
         ret = XNVCTRLQueryTargetStringAttribute
             (dpy, NV_CTRL_TARGET_TYPE_X_SCREEN, screen, 0,
-             NV_CTRL_STRING_TWINVIEW_XINERAMA_INFO_ORDER, &str);
+             NV_CTRL_STRING_NVIDIA_XINERAMA_INFO_ORDER, &str);
         
         if (!ret) {
-            fprintf(stderr, "Failed to query "
-                    "TwinViewXineramaInfoOrder.\n\n");
+            fprintf(stderr, "Failed to query nvidiaXineramaInfoOrder.\n\n");
             return 1;
         }
         
-        printf("TwinViewXineramaInfoOrder: %s\n\n", str);
+        printf("nvidiaXineramaInfoOrder: %s\n\n", str);
     }
     
     
     /*
-     * assign the TwinViewXineramaInfoOrder
+     * assign the nvidiaXineramaInfoOrder
      */
     
-    else if ((strcmp(argv[1], "--assign-twinview-xinerama-info-order")== 0)
+    else if ((strcmp(argv[1], "--assign-nvidia-xinerama-info-order")== 0)
              && argv[2]) {
         
         ret = XNVCTRLSetStringAttribute
             (dpy,
              screen,
              0,
-             NV_CTRL_STRING_TWINVIEW_XINERAMA_INFO_ORDER,
+             NV_CTRL_STRING_NVIDIA_XINERAMA_INFO_ORDER,
              argv[2]);
         
         if (!ret) {
             fprintf(stderr, "Failed to assign "
-                    "TwinViewXineramaInfoOrder = \"%s\".\n\n", argv[2]);
+                    "nvidiaXineramaInfoOrder = \"%s\".\n\n", argv[2]);
             return 1;
         }
         
-        printf("assigned TwinViewXineramaInfoOrder: \"%s\"\n\n",
+        printf("assigned nvidiaXineramaInfoOrder: \"%s\"\n\n",
                argv[2]);
     }
 
@@ -1152,7 +1237,7 @@ int main(int argc, char *argv[])
 
         /*
          * we have added a new MetaMode for this X screen, and we know
-         * it's id.  The last step is to use the XRandR extension to
+         * its id.  The last step is to use the XRandR extension to
          * switch to this mode; see the Xrandr(3) manpage for details.
          *
          * For demonstration purposes, just use the xrandr commandline
@@ -1163,7 +1248,54 @@ int main(int argc, char *argv[])
         printf("The id of the new MetaMode is %d; use xrandr to "
                "switch to it.\n\n", id);
     }
-    
+
+
+    /* Display all names each display device goes by
+     */
+    else if (strcmp(argv[1], "--print-display-names") == 0) {
+        unsigned int *pData;
+        int len, i;
+
+        printf("Display Device Information:\n");
+
+        ret = XNVCTRLQueryTargetBinaryData(dpy,
+                                           NV_CTRL_TARGET_TYPE_GPU,
+                                           0,
+                                           0,
+                                           NV_CTRL_BINARY_DATA_DISPLAY_TARGETS,
+                                           (unsigned char **) &pData,
+                                           &len);
+        if (!ret) {
+            fprintf(stderr, "Failed to query number of display devices.\n\n");
+            return 1;
+        }
+
+        printf("  number of display devices: %d\n", pData[0]);
+
+        for (i = 1; i <= pData[0]; i++) {
+
+            printf("\n  Display Device: %d\n", pData[i]);
+
+            print_display_name(dpy, pData[i],
+                               NV_CTRL_STRING_DISPLAY_NAME_TYPE_BASENAME,
+                               "Type Basename");
+            print_display_name(dpy, pData[i],
+                               NV_CTRL_STRING_DISPLAY_NAME_TYPE_ID,
+                               "Type ID");
+            print_display_name(dpy, pData[i],
+                               NV_CTRL_STRING_DISPLAY_NAME_DP_GUID,
+                               "DP GUID");
+            print_display_name(dpy, pData[i],
+                               NV_CTRL_STRING_DISPLAY_NAME_EDID_HASH,
+                               "EDID HASH");
+            print_display_name(dpy, pData[i],
+                               NV_CTRL_STRING_DISPLAY_NAME_TARGET_INDEX,
+                               "Target Index");
+            print_display_name(dpy, pData[i],
+                               NV_CTRL_STRING_DISPLAY_NAME_RANDR,
+                               "RANDR");
+        }
+    }
 
     /*
      * print help information
@@ -1204,6 +1336,9 @@ int main(int argc, char *argv[])
         printf("  --print-metamodes: print the current MetaModes for the "
                "X screen\n\n");
 
+        printf("  --print-metamodes-version2: print the current MetaModes for "
+               "the X screen with extended information\n\n");
+
         printf("  --add-metamode [metamode]: add the specified "
                "MetaMode to the X screen's list of MetaModes.\n\n");
         
@@ -1212,7 +1347,10 @@ int main(int argc, char *argv[])
 
         printf("  --print-current-metamode: print the current MetaMode.\n\n");
 
-        
+        printf("  --print-current-metamode-version2: print the current "
+               "MetaMode with extended information.\n\n");
+
+
         printf(" Misc options:\n\n");
         
         printf("  --get-valid-freq-ranges: query the valid frequency "
@@ -1232,11 +1370,11 @@ int main(int argc, char *argv[])
         
         printf("  --probe-dpys: probe GPUs for new display devices\n\n");
         
-        printf("  --query-twinview-xinerama-info-order: query the "
-               "TwinViewXineramaInfoOrder.\n\n");
+        printf("  --query-nvidia-xinerama-info-order: query the "
+               "nvidiaXineramaInfoOrder.\n\n");
         
-        printf("  --assign-twinview-xinerama-info-order [order]: assign the "
-               "TwinViewXineramaInfoOrder.\n\n");
+        printf("  --assign-nvidia-xinerama-info-order [order]: assign the "
+               "nvidiaXineramaInfoOrder.\n\n");
 
         printf("  --max-screen-size: query the maximum screen size "
                "on all GPUs in the system\n\n");
@@ -1246,6 +1384,9 @@ int main(int argc, char *argv[])
 
         printf("  --dynamic-twinview: demonstrates the process of "
                "dynamically transitioning into TwinView.\n\n");
+
+        printf("  --print-display-names: print all the names associated with "
+               "each display device on the server\n\n");
     }
 
     return 0;
@@ -1260,7 +1401,7 @@ int main(int argc, char *argv[])
 
 
 /*
- * display_device_name() - return the display device name correspoding
+ * display_device_name() - return the display device name corresponding
  * to the specified display device mask.
  */
 
