@@ -112,13 +112,6 @@ static AttrEntry attr_int_table[] = {
     MAKE_ENTRY(NV_CTRL_FLIPPING_ALLOWED),
     MAKE_ENTRY(NV_CTRL_ARCHITECTURE),
     MAKE_ENTRY(NV_CTRL_TEXTURE_CLAMPING),
-    MAKE_ENTRY(NV_CTRL_CURSOR_SHADOW),
-    MAKE_ENTRY(NV_CTRL_CURSOR_SHADOW_ALPHA),
-    MAKE_ENTRY(NV_CTRL_CURSOR_SHADOW_RED),
-    MAKE_ENTRY(NV_CTRL_CURSOR_SHADOW_GREEN),
-    MAKE_ENTRY(NV_CTRL_CURSOR_SHADOW_BLUE),
-    MAKE_ENTRY(NV_CTRL_CURSOR_SHADOW_X_OFFSET),
-    MAKE_ENTRY(NV_CTRL_CURSOR_SHADOW_Y_OFFSET),
     MAKE_ENTRY(NV_CTRL_FSAA_APPLICATION_CONTROLLED),
     MAKE_ENTRY(NV_CTRL_LOG_ANISO_APPLICATION_CONTROLLED),
     MAKE_ENTRY(NV_CTRL_IMAGE_SHARPENING),
@@ -142,7 +135,6 @@ static AttrEntry attr_int_table[] = {
     MAKE_ENTRY(NV_CTRL_GVIO_REQUESTED_VIDEO_FORMAT),
     MAKE_ENTRY(NV_CTRL_GVIO_DETECTED_VIDEO_FORMAT),
     MAKE_ENTRY(NV_CTRL_GVO_DATA_FORMAT),
-    MAKE_ENTRY(NV_CTRL_GVO_DISPLAY_X_SCREEN),
     MAKE_ENTRY(NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECTED),
     MAKE_ENTRY(NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECT_MODE),
     MAKE_ENTRY(NV_CTRL_GVO_SDI_SYNC_INPUT_DETECTED),
@@ -155,8 +147,6 @@ static AttrEntry attr_int_table[] = {
     MAKE_ENTRY(NV_CTRL_GVIO_VIDEO_FORMAT_WIDTH),
     MAKE_ENTRY(NV_CTRL_GVIO_VIDEO_FORMAT_HEIGHT),
     MAKE_ENTRY(NV_CTRL_GVIO_VIDEO_FORMAT_REFRESH_RATE),
-    MAKE_ENTRY(NV_CTRL_GVO_X_SCREEN_PAN_X),
-    MAKE_ENTRY(NV_CTRL_GVO_X_SCREEN_PAN_Y),
     MAKE_ENTRY(NV_CTRL_GPU_OVERCLOCKING_STATE),
     MAKE_ENTRY(NV_CTRL_GPU_2D_CLOCK_FREQS),
     MAKE_ENTRY(NV_CTRL_GPU_3D_CLOCK_FREQS),
@@ -353,7 +343,7 @@ static AttrEntry attr_str_table[] = {
     MAKE_ENTRY(NV_CTRL_STRING_VALID_HORIZ_SYNC_RANGES),
     MAKE_ENTRY(NV_CTRL_STRING_VALID_VERT_REFRESH_RANGES),
     MAKE_ENTRY(NV_CTRL_STRING_XINERAMA_SCREEN_INFO),
-    MAKE_ENTRY(NV_CTRL_STRING_TWINVIEW_XINERAMA_INFO_ORDER),
+    MAKE_ENTRY(NV_CTRL_STRING_NVIDIA_XINERAMA_INFO_ORDER),
     MAKE_ENTRY(NV_CTRL_STRING_SLI_MODE),
     MAKE_ENTRY(NV_CTRL_STRING_PERFORMANCE_MODES),
     MAKE_ENTRY(NV_CTRL_STRING_VCSC_FAN_STATUS),
@@ -372,13 +362,20 @@ static AttrEntry attr_str_table[] = {
     MAKE_ENTRY(NV_CTRL_STRING_3D_VISION_PRO_GLASSES_FIRMWARE_DATE_A),
     MAKE_ENTRY(NV_CTRL_STRING_3D_VISION_PRO_GLASSES_ADDRESS),
     MAKE_ENTRY(NV_CTRL_STRING_3D_VISION_PRO_GLASSES_NAME),
+    MAKE_ENTRY(NV_CTRL_STRING_CURRENT_METAMODE_VERSION_2),
+    MAKE_ENTRY(NV_CTRL_STRING_DISPLAY_NAME_TYPE_BASENAME),
+    MAKE_ENTRY(NV_CTRL_STRING_DISPLAY_NAME_TYPE_ID),
+    MAKE_ENTRY(NV_CTRL_STRING_DISPLAY_NAME_DP_GUID),
+    MAKE_ENTRY(NV_CTRL_STRING_DISPLAY_NAME_EDID_HASH),
+    MAKE_ENTRY(NV_CTRL_STRING_DISPLAY_NAME_TARGET_INDEX),
+    MAKE_ENTRY(NV_CTRL_STRING_DISPLAY_NAME_RANDR),
     { -1, NULL, NULL }
 };
 
 static AttrEntry attr_bin_table[] = {
     MAKE_ENTRY(NV_CTRL_BINARY_DATA_EDID),
     MAKE_ENTRY(NV_CTRL_BINARY_DATA_MODELINES),
-    MAKE_ENTRY(NV_CTRL_BINARY_DATA_METAMODES),
+    MAKE_ENTRY(NV_CTRL_BINARY_DATA_METAMODES_VERSION_1),
     MAKE_ENTRY(NV_CTRL_BINARY_DATA_XSCREENS_USING_GPU),
     MAKE_ENTRY(NV_CTRL_BINARY_DATA_GPUS_USED_BY_XSCREEN),
     MAKE_ENTRY(NV_CTRL_BINARY_DATA_GPUS_USING_FRAMELOCK),
@@ -390,6 +387,9 @@ static AttrEntry attr_bin_table[] = {
     MAKE_ENTRY(NV_CTRL_BINARY_DATA_GPUS_USED_BY_LOGICAL_XSCREEN),
     MAKE_ENTRY(NV_CTRL_BINARY_DATA_THERMAL_SENSORS_USED_BY_GPU),
     MAKE_ENTRY(NV_CTRL_BINARY_DATA_GLASSES_PAIRED_TO_3D_VISION_PRO_TRANSCEIVER),
+    MAKE_ENTRY(NV_CTRL_BINARY_DATA_DISPLAY_TARGETS),
+    MAKE_ENTRY(NV_CTRL_BINARY_DATA_DISPLAYS_CONNECTED_TO_GPU),
+    MAKE_ENTRY(NV_CTRL_BINARY_DATA_METAMODES_VERSION_2),
     { -1, NULL, NULL }
 };
 
@@ -404,7 +404,7 @@ static AttrEntry attr_strop_table[] = {
 };
 
 
-void print_perms(NVCTRLAttributePermissionsRec *perms)
+static void print_perms(NVCTRLAttributePermissionsRec *perms)
 {
     printf("%c", (perms->permissions & ATTRIBUTE_TYPE_READ)                      ? 'R' : '_');
     printf("%c", (perms->permissions & ATTRIBUTE_TYPE_WRITE)                     ? 'W' : '_');
@@ -426,7 +426,7 @@ void print_perms(NVCTRLAttributePermissionsRec *perms)
 case (FMT):                  \
     return #FMT;
 
-const char *GetAttrTypeName(value)
+static const char *GetAttrTypeName(int value)
 {
     switch (value) {
         ADD_NVCTRL_CASE(ATTRIBUTE_TYPE_UNKNOWN);
