@@ -109,7 +109,7 @@ static const char * __cav_help =
   "cav (Caveats) - Caveats for this configuration.  A frame buffer "
   "configuration may have the following caveats: 'NonC' if it supports "
   "any non-conformant visual extension.  'Slow' if it has reduced "
-  "preformance.  '-' if it has no caveats.";
+  "performance.  '-' if it has no caveats.";
 static const char * __pbw_help =
   "pbw (Pbuffer width) - Width of pbuffer (in hexadecimal).";
 static const char * __pbh_help =
@@ -120,15 +120,15 @@ static const char * __pbp_help =
 static const char * __trt_help =
   "trt (Transparency type) - Type of transparency (RGBA or Index).";
 static const char * __trr_help =
-  "trr (Transparency red value) - Red value consided transparent.";
+  "trr (Transparency red value) - Red value considered transparent.";
 static const char * __trg_help =
-  "trg (Transparency green value) - Green value consided transparent.";
+  "trg (Transparency green value) - Green value considered transparent.";
 static const char * __trb_help =
-  "trb (Transparency blue value) - Blue value consided transparent.";
+  "trb (Transparency blue value) - Blue value considered transparent.";
 static const char * __tra_help =
-  "tra (Transparency alpha value) - Alpha value consided transparent.";
+  "tra (Transparency alpha value) - Alpha value considered transparent.";
 static const char * __tri_help =
-  "tri (Transparency index value) - Color index value consided transparent.";
+  "tri (Transparency index value) - Color index value considered transparent.";
 
 
 GType ctk_glx_get_type(void)
@@ -146,6 +146,7 @@ GType ctk_glx_get_type(void)
             sizeof (CtkGLX),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_glx_type = g_type_register_static (GTK_TYPE_VBOX,
@@ -232,7 +233,7 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
     GtkWidget *banner;
     GtkWidget *hseparator;
     GtkWidget *hbox;
-    GtkWidget *vbox, *vbox2;
+    GtkWidget *vbox;
     GtkWidget *scrollWin;
     GtkWidget *event;    /* For setting the background color to white */
     GtkWidget *data_table, *header_table;
@@ -368,7 +369,6 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
 
     hbox      = gtk_hbox_new(FALSE, 0);
     vbox      = gtk_vbox_new(FALSE, 10);
-    vbox2     = gtk_vbox_new(FALSE, 10);
 
     data_viewport = gtk_viewport_new(NULL, NULL);
     gtk_widget_set_size_request(data_viewport, 400, 50);
@@ -945,7 +945,7 @@ GtkTextBuffer *ctk_glx_create_help(GtkTextTagTable *table,
                  );
 
     ctk_help_heading(b, &i, "Show GLX Frame Buffer Configurations");
-    ctk_help_para(b, &i, __show_fbc_help);
+    ctk_help_para(b, &i, "%s", __show_fbc_help);
 
     ctk_help_heading(b, &i, "GLX Frame Buffer Configurations");
     ctk_help_para(b, &i, "This table lists the supported frame buffer "
