@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -40,8 +35,10 @@
 
 static void probe_displays_received(GtkObject *object, gpointer arg1,
                                     gpointer user_data);
+static gboolean update_gpu_memory_used(gpointer);
+
 #define ARRAY_ELEMENTS 16
-#define DEFAULT_UPDATE_PCIE_INFO_TIME_INTERVAL 1000
+#define DEFAULT_UPDATE_GPU_INFO_TIME_INTERVAL 3000
 
 GType ctk_gpu_get_type(
     void
@@ -60,6 +57,7 @@ GType ctk_gpu_get_type(
             sizeof (CtkGpu),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_gpu_type =
@@ -71,57 +69,14 @@ GType ctk_gpu_get_type(
 }
 
 
-static gchar *make_display_device_list(NvCtrlAttributeHandle *handle,
-                                       unsigned int display_devices)
+static gchar *make_display_device_list(NvCtrlAttributeHandle *handle)
 {
-    gchar *displays = NULL;
-    gchar *type;
-    gchar *name;
-    gchar *tmp_str;
-    unsigned int mask;
-    ReturnStatus ret;
-    
-
-    /* List of Display Device connected on GPU */
-
-    for (mask = 1; mask; mask <<= 1) {
-        
-        if (!(mask & display_devices)) continue;
-        
-        type = display_device_mask_to_display_device_name(mask);
-        name = NULL;
-        
-        ret =
-            NvCtrlGetStringDisplayAttribute(handle,
-                                            mask,
-                                            NV_CTRL_STRING_DISPLAY_DEVICE_NAME,
-                                            &name);
-        if (ret != NvCtrlSuccess) {
-            tmp_str = g_strdup_printf("Unknown (%s)", type);
-        } else {
-            tmp_str = g_strdup_printf("%s (%s)", name, type);
-            XFree(name);
-        }
-        free(type);
-        
-        if (displays) {
-            name = g_strdup_printf("%s,\n%s", tmp_str, displays);
-            g_free(displays);
-            g_free(tmp_str);
-        } else {
-            name = tmp_str;
-        }
-        displays = name;
-    }
-
-    if (!displays) {
-        displays = g_strdup("None");
-    }
-
-    return displays;
+    return create_display_name_list_string(handle,
+                                           NV_CTRL_BINARY_DATA_DISPLAYS_CONNECTED_TO_GPU);
 
 } /* make_display_device_list() */
 
+
 void get_bus_type_str(NvCtrlAttributeHandle *handle,
                       gchar **bus)
 {
@@ -142,12 +97,12 @@ void get_bus_type_str(NvCtrlAttributeHandle *handle,
             bus_type_str = "Integrated";
     }
 
-    /* NV_CTRL_BUS_RATE */
+    /* NV_CTRL_GPU_PCIE_MAX_LINK_WIDTH */
 
     bus_rate = NULL;
     if (bus_type == NV_CTRL_BUS_TYPE_AGP ||
         bus_type == NV_CTRL_BUS_TYPE_PCI_EXPRESS) {
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_BUS_RATE, &tmp);
+        ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_PCIE_MAX_LINK_WIDTH, &tmp);
         if (ret == NvCtrlSuccess) {
             if (bus_type == NV_CTRL_BUS_TYPE_PCI_EXPRESS) {
                 bus_rate = g_strdup_printf("x%u", tmp);
@@ -161,9 +116,7 @@ void get_bus_type_str(NvCtrlAttributeHandle *handle,
 
     pcie_gen = NULL;
     if (bus_type == NV_CTRL_BUS_TYPE_PCI_EXPRESS) {
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_PCIE_GENERATION, &tmp);
-        if (ret == NvCtrlSuccess)
-            pcie_gen = g_strdup_printf("Gen%u", tmp);
+        pcie_gen = get_pcie_generation_string(handle);
     }
 
     /* concatenate all the available bus related information */
@@ -220,33 +173,6 @@ void get_bus_id_str(NvCtrlAttributeHandle *handle,
     *pci_bus_id = bus_id;
 }
 
-static gboolean update_pcie_info(gpointer user_data)
-{
-    gchar *bus;
-    gint link_speed;
-    gchar *link_speed_str;
-    ReturnStatus ret;
-
-    CtkGpu *ctk_gpu = (CtkGpu *) user_data;
-    get_bus_type_str(ctk_gpu->handle, &bus);
-    gtk_label_set_text(GTK_LABEL(ctk_gpu->bus_label), bus);
-    
-    if (ctk_gpu->pcie_gen_queriable) {
-        ret = NvCtrlGetAttribute(ctk_gpu->handle, NV_CTRL_GPU_PCIE_MAX_LINK_SPEED,
-                                 &link_speed);
-        if (ret != NvCtrlSuccess) {
-            link_speed_str = g_strdup_printf("Unknown");
-        }
-        else {
-            link_speed_str = g_strdup_printf("%d", link_speed);
-        }
-        gtk_label_set_text(GTK_LABEL(ctk_gpu->link_speed_label), link_speed_str);
-        g_free(link_speed_str);
-    }
-    g_free(bus);
-    return TRUE;
-}
-
 
 
 GtkWidget* ctk_gpu_new(
@@ -265,8 +191,7 @@ GtkWidget* ctk_gpu_new(
     GtkWidget *hseparator;
     GtkWidget *table;
 
-    char *product_name, *vbios_version, *video_ram, *irq;
-    gchar *s;
+    char *product_name, *vbios_version, *video_ram, *gpu_memory_text, *irq;
     gchar *pci_bus_id;
     gchar pci_device_id[ARRAY_ELEMENTS];
     gchar pci_vendor_id[ARRAY_ELEMENTS];
@@ -280,14 +205,18 @@ GtkWidget* ctk_gpu_new(
     gchar *tmp_str;
     gchar *gpu_cores;
     gchar *memory_interface;
+    gchar *bus = NULL;
+    gchar *link_speed_str = NULL;
+    gchar *link_width_str = NULL;
+    gchar *pcie_gen_str = NULL;
 
-    unsigned int display_devices;
     int xinerama_enabled;
     int *pData;
     int len;
     int i;
     int row = 0;
-    int total_rows = 19;
+    int total_rows = 21;
+    int gpu_memory;
 
 
     /*
@@ -343,6 +272,16 @@ GtkWidget* ctk_gpu_new(
     } else {
         video_ram = g_strdup_printf("%d MB", tmp >> 10);
     }
+
+    /* NV_CTRL_TOTAL_DEDICATED_GPU_MEMORY */
+
+    ret = NvCtrlGetAttribute(handle, NV_CTRL_TOTAL_DEDICATED_GPU_MEMORY, 
+                             &gpu_memory);
+    if (ret != NvCtrlSuccess) {
+        gpu_memory_text = NULL;
+    } else {
+        gpu_memory_text = g_strdup_printf("%d MB", gpu_memory);
+    }
     
     /* NV_CTRL_GPU_CORES */
 
@@ -425,17 +364,8 @@ GtkWidget* ctk_gpu_new(
         screens = g_strdup("Unknown");
     }
 
-    /* List of Display Device connected on GPU */
-
-    displays = NULL;
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_CONNECTED_DISPLAYS,
-                             (int *)&display_devices);
-    if (ret == NvCtrlSuccess) {
-        displays = make_display_device_list(handle, display_devices);
-    }
-    
     /* now, create the object */
-    
+
     object = g_object_new(CTK_TYPE_GPU, NULL);
     ctk_gpu = CTK_GPU(object);
 
@@ -445,7 +375,9 @@ GtkWidget* ctk_gpu_new(
     ctk_gpu->gpu_cores = (gpu_cores != NULL) ? 1 : 0;
     ctk_gpu->memory_interface = (memory_interface != NULL) ? 1 : 0;
     ctk_gpu->ctk_config = ctk_config;
+    ctk_gpu->ctk_event = ctk_event;
     ctk_gpu->pcie_gen_queriable = FALSE;
+    ctk_gpu->gpu_memory = gpu_memory;
 
     /* set container properties of the object */
 
@@ -456,16 +388,22 @@ GtkWidget* ctk_gpu_new(
     banner = ctk_banner_image_new(BANNER_ARTWORK_GPU);
     gtk_box_pack_start(GTK_BOX(ctk_gpu), banner, FALSE, FALSE, 0);
 
-    /* NV_CTRL_GPU_PCIE_GENERATION */
-
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_PCIE_GENERATION, &tmp);
-    if (ret == NvCtrlSuccess)
+    /* PCIe link information */
+    get_bus_type_str(handle, &bus);
+    pcie_gen_str = get_pcie_generation_string(handle);
+    if (pcie_gen_str) {
         ctk_gpu->pcie_gen_queriable = TRUE;
+        link_speed_str =
+            get_pcie_link_speed_string(ctk_gpu->handle,
+                                       NV_CTRL_GPU_PCIE_MAX_LINK_SPEED);
+        link_width_str = get_pcie_link_width_string(ctk_gpu->handle,
+                                                    NV_CTRL_GPU_PCIE_MAX_LINK_WIDTH);
+    }
 
     /*
      * GPU information: TOP->MIDDLE - LEFT->RIGHT
      *
-     * This displays basic display adatper information, including
+     * This displays basic display adapter information, including
      * product name, bios version, bus type, video ram and interrupt
      * line.
      */
@@ -501,8 +439,16 @@ GtkWidget* ctk_gpu_new(
                   0, 0.5, "VBIOS Version:",
                   0, 0.5, vbios_version);
     add_table_row(table, row++,
-                  0, 0.5, "Memory:",
+                  0, 0.5, "Total Memory:",
                   0, 0.5, video_ram);
+    add_table_row(table, row++,
+                  0, 0.5, "Total Dedicated Memory:",
+                  0, 0.5, gpu_memory_text);
+    ctk_gpu->gpu_memory_used_label = 
+        add_table_row(table, row++,
+                      0, 0.5, "Used Dedicated Memory:",
+                      0, 0.5, NULL);
+    update_gpu_memory_used(ctk_gpu);
     if ( ctk_gpu->memory_interface ) {
         gtk_table_resize(GTK_TABLE(table), ++total_rows, 2);
         add_table_row(table, row++,
@@ -511,10 +457,9 @@ GtkWidget* ctk_gpu_new(
     }
     /* spacing */
     row += 3;
-    ctk_gpu->bus_label =
-        add_table_row(table, row++,
-                      0, 0.5, "Bus Type:",
-                      0, 0.5, NULL);
+    add_table_row(table, row++,
+                  0, 0.5, "Bus Type:",
+                  0, 0.5, bus);
     add_table_row(table, row++,
                   0, 0.5, "Bus ID:",
                   0, 0.5, pci_bus_id);
@@ -530,10 +475,18 @@ GtkWidget* ctk_gpu_new(
     if (ctk_gpu->pcie_gen_queriable) { 
         /* spacing */
         row += 3;
-        ctk_gpu->link_speed_label =
-            add_table_row(table, row++,
-                          0, 0.5, "PCI-E Max Link Speed:",
-                          0, 0.5, NULL);
+        add_table_row(table, row++,
+                      0, 0.5, "PCIe Generation:",
+                      0, 0.5, pcie_gen_str);
+        add_table_row(table, row++,
+                      0, 0.5, "Maximum PCIe Link Width:",
+                      0, 0.5, link_width_str);
+        add_table_row(table, row++,
+                      0, 0.5, "Maximum PCIe Link Speed:",
+                      0, 0.5, link_speed_str);
+        g_free(link_speed_str);
+        g_free(link_width_str);
+        g_free(pcie_gen_str);
         row++;
     }
 
@@ -543,6 +496,8 @@ GtkWidget* ctk_gpu_new(
                   0, 0, "X Screens:",
                   0, 0, screens);
     /* spacing */
+    displays = make_display_device_list(handle);
+
     row += 3;
     ctk_gpu->displays =
         add_table_row(table, row,
@@ -558,23 +513,11 @@ GtkWidget* ctk_gpu_new(
     g_free(irq);
     g_free(screens);
     g_free(displays);
+    g_free(bus);
+    g_free(gpu_memory_text);
 
     gtk_widget_show_all(GTK_WIDGET(object));
     
-    /* update PCI-E info */
-    update_pcie_info((gpointer) ctk_gpu);
-
-    /* Register a timer callback to update the PCI-E info */
-    s = g_strdup_printf("Graphics Card (GPU %d)",
-                        NvCtrlGetTargetId(handle));
-
-    ctk_config_add_timer(ctk_config,
-                         DEFAULT_UPDATE_PCIE_INFO_TIME_INTERVAL,
-                         s,
-                         (GSourceFunc) update_pcie_info,
-                         (gpointer) ctk_gpu);
-    g_free(s);
-
     /* Handle events */
     
     g_signal_connect(G_OBJECT(ctk_event),
@@ -582,6 +525,16 @@ GtkWidget* ctk_gpu_new(
                      G_CALLBACK(probe_displays_received),
                      (gpointer) ctk_gpu);
 
+    tmp_str = g_strdup_printf("Memory Used (GPU %d)",
+                              NvCtrlGetTargetId(handle));
+
+    ctk_config_add_timer(ctk_gpu->ctk_config,
+                         DEFAULT_UPDATE_GPU_INFO_TIME_INTERVAL,
+                         tmp_str,
+                         (GSourceFunc) update_gpu_memory_used,
+                         (gpointer) ctk_gpu);
+    g_free(tmp_str);
+
     return GTK_WIDGET(object);
 }
 
@@ -615,7 +568,7 @@ GtkTextBuffer *ctk_gpu_create_help(GtkTextTagTable *table,
     ctk_help_heading(b, &i, "VBIOS Version");
     ctk_help_para(b, &i, "This is the Video BIOS version.");
     
-    ctk_help_heading(b, &i, "Memory"); 
+    ctk_help_heading(b, &i, "Total Memory"); 
     ctk_help_para(b, &i, "This is the overall amount of memory "
                   "available to your GPU.  With TurboCache(TM) GPUs, "
                   "this value may exceed the amount of video "
@@ -624,6 +577,14 @@ GtkTextBuffer *ctk_gpu_create_help(GtkTextTagTable *table,
                   "dedicated system memory set aside by the system "
                   "BIOS for use by the integrated GPU.");
 
+    ctk_help_heading(b, &i, "Total Dedicated Memory"); 
+    ctk_help_para(b, &i, "This is the amount of memory dedicated "
+                  "exclusively to your GPU.");
+
+    ctk_help_heading(b, &i, "Used Dedicated Memory"); 
+    ctk_help_para(b, &i, "This is the amount of dedicated memory used "
+                  "by your GPU.");
+
     if (ctk_gpu->memory_interface) {
         ctk_help_heading(b, &i, "Memory Interface");
         ctk_help_para(b, &i, "This is the bus bandwidth of the GPU's "
@@ -641,7 +602,7 @@ GtkTextBuffer *ctk_gpu_create_help(GtkTextTagTable *table,
                   "in X configuration file 'BusID' format: "
                   "\"bus:device:function\", or, if the PCI domain of the GPU "
                   "is non-zero, \"bus@domain:device:function\".  Note "
-                  "that all values are in decimal (as opposed to hexidecimal, "
+                  "that all values are in decimal (as opposed to hexadecimal, "
                   "which is how `lspci` formats its BusID values).");
 
     ctk_help_heading(b, &i, "PCI Device ID");
@@ -653,10 +614,27 @@ GtkTextBuffer *ctk_gpu_create_help(GtkTextTagTable *table,
     ctk_help_heading(b, &i, "IRQ");
     ctk_help_para(b, &i, "This is the interrupt request line assigned to "
                   "this GPU.");
-    
+
     if (ctk_gpu->pcie_gen_queriable) {
-        ctk_help_heading(b, &i, "PCI-E Max Link Speed");
-        ctk_help_para(b, &i, "This is the maximum PCI-E link speed.");
+        ctk_help_heading(b, &i, "PCIe Generation");
+        ctk_help_para(b, &i, "This is the PCIe generation that this GPU, in "
+                      "this system, is compliant with.");
+
+        ctk_help_heading(b, &i, "Maximum PCIe Link Width");
+        ctk_help_para(b, &i, "This is the maximum width that the PCIe link "
+                      "between the GPU and the system may be trained to.  This "
+                      "is expressed in number of lanes.  The trained link "
+                      "width may vary dynamically and possibly be narrower "
+                      "based on the GPU's utilization and performance "
+                      "settings.");
+
+        ctk_help_heading(b, &i, "Maximum PCIe Link Speed");
+        ctk_help_para(b, &i, "This is the maximum speed that the PCIe link "
+                      "between the GPU and the system may be trained to.  "
+                      "This is expressed in gigatransfers per second "
+                      "(GT/s).  The link may be dynamically trained to a "
+                      "slower speed, based on the GPU's utilization and "
+                      "performance settings."); 
     }
     
     ctk_help_heading(b, &i, "X Screens");
@@ -676,37 +654,70 @@ GtkTextBuffer *ctk_gpu_create_help(GtkTextTagTable *table,
 static void probe_displays_received(GtkObject *object, gpointer arg1,
                                     gpointer user_data)
 {
-    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkGpu *ctk_object = CTK_GPU(user_data);
-    unsigned int probed_displays = event_struct->value;
     gchar *str;
 
-    str = make_display_device_list(ctk_object->handle, probed_displays);
+    str = make_display_device_list(ctk_object->handle);
 
     gtk_label_set_text(GTK_LABEL(ctk_object->displays), str);
 
     g_free(str);
 }
 
-void ctk_gpu_start_timer(GtkWidget *widget)
+static gboolean update_gpu_memory_used(gpointer user_data)
+{
+    CtkGpu *ctk_gpu;
+    gchar *memory_text;
+    ReturnStatus ret;
+    int value;
+    static int num_failures = 0;
+
+    ctk_gpu = CTK_GPU(user_data);
+
+    ret = NvCtrlGetAttribute(ctk_gpu->handle, 
+                             NV_CTRL_USED_DEDICATED_GPU_MEMORY, 
+                             &value);
+    if (ret != NvCtrlSuccess || value > ctk_gpu->gpu_memory || value < 0) {
+        gtk_label_set_text(GTK_LABEL(ctk_gpu->gpu_memory_used_label), "Unknown");
+        if(num_failures++ >= 10) {
+            return FALSE;
+        }
+    } else {
+        if (ctk_gpu->gpu_memory > 0) {
+            memory_text = g_strdup_printf("%d MB (%.0f%%)", 
+                                          value, 
+                                          100.0 * (double) value / 
+                                              (double) ctk_gpu->gpu_memory);
+        } else {
+            memory_text = g_strdup_printf("%d MB", value);
+        }
+
+        gtk_label_set_text(GTK_LABEL(ctk_gpu->gpu_memory_used_label), memory_text);
+        g_free(memory_text);
+    }
+
+    return TRUE;
+}
+
+void ctk_gpu_page_select(GtkWidget *widget)
 {
     CtkGpu *ctk_gpu = CTK_GPU(widget);
 
-    /* Start the GPU timer */
+    /* Start the gpu timer */
 
     ctk_config_start_timer(ctk_gpu->ctk_config,
-                           (GSourceFunc) update_pcie_info,
+                           (GSourceFunc) update_gpu_memory_used,
                            (gpointer) ctk_gpu);
 }
 
-void ctk_gpu_stop_timer(GtkWidget *widget)
+void ctk_gpu_page_unselect(GtkWidget *widget)
 {
     CtkGpu *ctk_gpu = CTK_GPU(widget);
 
-    /* Stop the GPU timer */
+    /* Stop the gpu timer */
 
     ctk_config_stop_timer(ctk_gpu->ctk_config,
-                          (GSourceFunc) update_pcie_info,
+                          (GSourceFunc) update_gpu_memory_used,
                           (gpointer) ctk_gpu);
 }
 
